from django.conf.urls import url, include

from .views import api_v1_router, MeViewSet

urlpatterns = (
    url(r"^me/logout/?$", "django.contrib.auth.views.logout", name="logout"),
    url(r'^me/?$', MeViewSet.as_view({'get': 'retrieve'}), name='me'),
    url(r"^", include(api_v1_router.urls))       # noqa
)



from django.contrib import auth
from django.core.exceptions import ValidationError
from django.db import transaction
from django.template.defaultfilters import slugify
from firebase_token_generator import create_token
from django.conf import settings


from rest_framework import serializers
from rest_framework import relations

from elastimorphic.serializers import ContentTypeField, PolymorphicSerializerMixin

from .models import Content, Tag, LogEntry, FeatureType


class ImageFieldSerializer(serializers.WritableField):

    def __init__(self, caption_field=None, alt_field=None, **kwargs):
        super(ImageFieldSerializer, self).__init__(**kwargs)

        self.caption_field = caption_field
        self.alt_field = alt_field

    def to_native(self, obj):
        if obj is None or obj.id is None:
            return None

        data = {
            "id": obj.id,
        }
        if self.caption_field:
            data["alt"] = obj.alt
            data["caption"] = obj.caption
        return data

    def from_native(self, data):
        if data is None:
            return None

        image_id = data.get("id")
        if image_id is not None:
            return int(image_id)  # Just in case a string gets passed in

        return None

    def field_from_native(self, data, files, field_name, into):
        super(ImageFieldSerializer, self).field_from_native(data, files, field_name, into)
        image_data = data.get(field_name, {})
        if image_data is None:
            return

        if self.alt_field and "alt" in image_data:
            into[self.alt_field] = image_data["alt"]

        if self.caption_field and "caption" in image_data:
            into[self.caption_field] = image_data["caption"]


class TagSerializer(serializers.ModelSerializer):
    class Meta:
        model = Tag

    def to_native(self, obj):
        return {
            "id": obj.pk,
            "name": obj.name,
            "slug": obj.slug,
            "type": obj.get_mapping_type_name()
        }


class TagField(relations.RelatedField):
    """This is a relational field that handles the addition of tags to content
    objects. This field also allows the user to create tags in the db if they
    don't already exist."""

    read_only = False

    def to_native(self, obj):
        return {
            "id": obj.pk,
            "name": obj.name,
            "slug": obj.slug,
            "type": obj.get_mapping_type_name()
        }

    def from_native(self, value):
        """Basically, each tag dict must include a full dict with id,
        name and slug--or else you need to pass in a dict with just a name,
        which indicated that the Tag doesn't exist, and should be added."""

        if "id" in value:
            tag = Tag.objects.get(id=value["id"])
        else:
            if "name" not in value:
                raise ValidationError("Tags must include an ID or a name.")
            name = value["name"]
            slug = value.get("slug", slugify(name))
            try:
                tag = Tag.objects.get(slug=slug)
            except Tag.DoesNotExist:
                tag, created = Tag.objects.get_or_create(name=name, slug=slug)
        return tag


class FeatureTypeField(relations.RelatedField):
    """This is a relational field that handles the addition of feature_types to
    content objects. This field also allows the user to create feature_types in
    the db if they don't already exist."""

    read_only = False

    def to_native(self, obj):
        return obj.name

    def from_native(self, value):
        """Basically, each tag dict must include a full dict with id,
        name and slug--or else you need to pass in a dict with just a name,
        which indicated that the Tag doesn't exist, and should be added."""
        if value == "":
            return None

        slug = slugify(value)
        feature_type, created = FeatureType.objects.get_or_create(
            slug=slug,
            defaults={"name": value}
        )
        return feature_type


class UserSerializer(serializers.ModelSerializer):
    """Returns basic User fields"""

    class Meta:
        model = auth.get_user_model()

    def to_native(self, obj):

        json = {
            "id": obj.pk,
            "username": obj.username,
            "email": obj.email,
            "first_name": obj.first_name,
            "last_name": obj.last_name,
            "full_name": obj.get_full_name(),
            "short_name": obj.get_short_name()
        }

        # ensure this variable exists
        secret = getattr(settings, 'FIREBASE_SECRET', None)
        if secret:
            # use firebase auth to provide auth variables to firebase security api
            firebase_auth_payload = {
                'id': obj.pk,
                'username': obj.username,
                'email': obj.email,
                'is_staff': obj.is_staff
            }
            json['firebase_token'] = create_token(secret, firebase_auth_payload)

        return json


class AuthorField(relations.RelatedField):
    """This field manages the authors on a piece of content, and allows a "fatter"
    endpoint then would normally be possible with a RelatedField"""

    read_only = False

    def to_native(self, obj):
        return {
            "id": obj.pk,
            "first_name": obj.first_name,
            "last_name": obj.last_name,
            "username": obj.username
        }

    def from_native(self, value):
        """Basically, each author dict must include either a username or id."""
        model = auth.get_user_model()

        if "id" in value:
            author = model.objects.get(id=value["id"])
        else:
            if "username" not in value:
                raise ValidationError("Authors must include an ID or a username.")
            username = value["username"]
            author = model.objects.get(username=username)
        return author


class ContentSerializer(serializers.ModelSerializer):
    polymorphic_ctype = ContentTypeField(source="polymorphic_ctype_id", read_only=True)
    tags = TagField(many=True)
    feature_type = FeatureTypeField(required=False)
    authors = AuthorField(many=True)
    thumbnail = ImageFieldSerializer(required=False)
    absolute_url = serializers.Field(source="get_absolute_url")
    status = serializers.Field(source="get_status")

    class Meta:
        model = Content

    @transaction.commit_on_success
    def save(self, *args, **kwargs):
        if not "index" in kwargs:
            kwargs["index"] = False
        return super(ContentSerializer, self).save(*args, **kwargs)


class LogEntrySerializer(serializers.ModelSerializer):
    class Meta:
        model = LogEntry


class PolymorphicContentSerializer(PolymorphicSerializerMixin, ContentSerializer):
    pass



from django.views.generic import ListView

from bulbs.content.models import Content


class ContentListView(ListView):

    feature_types = None
    tags = None
    types = None
    published = None
    authors = None

    allow_empty = True
    paginate_by = 20
    context_object_name = 'content_list'
    template_name = None

    def get_queryset(self):
        search_kwargs = {}
        if 'tags' in self.request.GET:
            search_kwargs['tags'] = self.request.GET.getlist('tags', [])
        elif 'tag' in self.request.GET:
            search_kwargs['tags'] = self.request.GET.getlist('tag', [])

        if 'tags' in self.kwargs:
            tags = self.kwargs['tags']
            if not isinstance(tags, list):
                tags = [tags]
            search_kwargs['tags'] = tags
        if self.tags > 0:
            search_kwargs['tags'] = self.tags

        if 'types' in self.request.GET:
            search_kwargs['types'] = self.request.GET.getlist('types', [])
        elif 'type' in self.request.GET:
            search_kwargs['types'] = self.request.GET.getlist('type', [])

        if 'types' in self.kwargs:
            search_kwargs['types'] = self.kwargs['types']
        if self.types > 0:
            search_kwargs['types'] = self.types

        if 'feature_types' in self.request.GET:
            search_kwargs['feature_types'] = self.request.GET.getlist('feature_types', [])
        elif 'feature_type' in self.request.GET:
            search_kwargs['feature_types'] = self.request.GET.getlist('feature_type', [])

        if 'feature_types' in self.kwargs:
            search_kwargs['feature_types'] = self.kwargs['feature_types']
        if self.feature_types > 0:
            search_kwargs['feature_types'] = self.feature_types

        if 'published' in self.kwargs:
            search_kwargs['published'] = self.kwargs['published']
        if self.published:
            search_kwargs['published'] = self.published

        if 'authors' in self.request.GET:
            search_kwargs['authors'] = self.request.GET.getlist('authors', [])
        elif 'author' in self.request.GET:
            search_kwargs['authors'] = self.request.GET.getlist('author', [])
        if 'authors' in self.kwargs:
            search_kwargs['authors'] = self.kwargs['authors']
        if self.authors:
            search_kwargs['authors'] = self.authors

        if 'q' in self.request.GET:
            search_kwargs['query'] = self.request.GET['q']

        return Content.search_objects.search(**search_kwargs)


content_list = ContentListView.as_view()



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Video'
        db.create_table(u'videos_video', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('data', self.gf('json_field.fields.JSONField')(default=u'null', null=True, blank=True)),
            ('sources', self.gf('json_field.fields.JSONField')(default=[], null=True, blank=True)),
            ('poster_url', self.gf('django.db.models.fields.URLField')(max_length=200, null=True, blank=True)),
            ('status', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('job_id', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
        ))
        db.send_create_signal(u'videos', ['Video'])


    def backwards(self, orm):
        # Deleting model 'Video'
        db.delete_table(u'videos_video')


    models = {
        u'videos.video': {
            'Meta': {'object_name': 'Video'},
            'data': ('json_field.fields.JSONField', [], {'default': "u'null'", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'job_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'poster_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'sources': ('json_field.fields.JSONField', [], {'default': '[]', 'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.IntegerField', [], {'default': '0'})
        }
    }

    complete_apps = ['videos']


from django.db import models
from django.utils import timezone

from json_field import JSONField

from bulbs.content.models import Content
from .operations import *  # noqa


class ContentListManager(models.Manager):
    
    def preview(self, name, when):
        content_list = self.get(name=name)
        data = content_list.data
        for operation in content_list.operations.filter(when__lte=when, applied=False):
            data = operation.apply(data)

        content_list.data = data
        return content_list

    def applied(self, name):
        content_list = self.get(name=name)
        data = content_list.data
        for operation in content_list.operations.filter(when__lte=timezone.now(), applied=False):
            data = operation.apply(data)
            operation.applied = True

        content_list.data = data
        content_list.save()
        return content_list


class ContentList(models.Model):
    name = models.SlugField(unique=True)
    length = models.IntegerField(default=10)
    data = JSONField(default=[])

    objects = ContentListManager()

    def __len__(self):
        return min(self.length, len(self.data))

    def __iter__(self):
        content_ids = [item["id"] for item in self.data[:self.__len__()]]
        bulk = Content.objects.in_bulk(content_ids)
        for pk in content_ids:
            yield bulk.get(pk)

    def __getitem__(self, index):
        items = self.data[:self.__len__()].__getitem__(index)
        if isinstance(items, dict):
            return Content.objects.get(id=self.data[index]["id"])
        if isinstance(items, list):
            content = []
            content_ids = [item["id"] for item in items]
            bulk = Content.objects.in_bulk(content_ids)
            for pk in content_ids:
                content.append(bulk.get(pk))
            return content
        raise IndexError("Index out of range")

    def __setitem__(self, index, value):
        if index > self.__len__():
            raise IndexError("Index out of range")
        if isinstance(value, Content):
            self.data[index]["id"] = value.pk
        elif isinstance(value, int):
            self.data[index]["id"] = value
        else:
            raise ValueError("ContentList items must be Content or int")

    def __delitem__(self, index):
        if index > self.__len__():
            raise IndexError("Index out of range")
        del self.data[index]

    def __contains__(self, value):
        if isinstance(value, Content):
            value = value.pk
        if isinstance(value, int):
            for item in self.data:
                if value == item["id"]:
                    return True
        return False

    def __unicode__(self):
        return "{}[{}]".format(self.name, self.__len__())


class ContentListHistory(models.Model):
    content_list = models.ForeignKey(ContentList, related_name="history")
    data = JSONField(default=[])
    date = models.DateTimeField(auto_now_add=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import re
import os
import sys


name = "django-bulbs"
package = "bulbs"
description = "America's Finest Namespace"
url = "https://github.com/theonion/django-bulbs"
author = "Chris Sinchok"
author_email = "csinchok@theonion.com"
license = "BSD"
requires = [
    "Django>=1.4,<1.7",
    "South==0.8.1",
    "django-betty-cropper>=0.1.4",
    "django-elastimorphic>=0.1.0",
    "django-json-field==0.5.5",
    "django_polymorphic==0.5.1",
    "djangorestframework==2.3.13",
    "python-dateutil==2.1",
    "pytz==2012h",
    "requests>=1.1.0",
    "simplejson==3.3.0",
    "six==1.6.1",
    "firebase-token-generator==1.3.2"
]


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, "__init__.py")).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, "__init__.py"))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, "", 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, "__init__.py"))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    args = {"version": get_version(package)}
    print "You probably want to also tag the version now:"
    print "  git tag -a %(version)s -m 'version %(version)s'" % args
    print "  git push --tags"
    sys.exit()


setup(
    name=name,
    version=get_version(package),
    url=url,
    license=license,
    description=description,
    author=author,
    author_email=author_email,
    packages=get_packages(package),
    package_data=get_package_data(package),
    install_requires=requires
)
